@@ -6,9 +6,12 @@
 # the libcurl tutorial for more info.
 try:
     import signal
-    signal.signal(signal.SIGPIPE, signal.SIG_IGN)
+    from signal import SIGPIPE, SIG_IGN
 except ImportError:
     pass
+else:
+    signal.signal(SIGPIPE, SIG_IGN)
+
 try:
     from cStringIO import StringIO
 except ImportError:
