@@ -13,16 +13,16 @@ setup_module, teardown_module = appmanager.setup(('app', 8383, dict(ssl=True)))
 
 class CertinfoTest(unittest.TestCase):
     def setUp(self):
-        self.curl = pycurl.Curl()
-    
+        self.curl = util.DefaultCurl()
+
     def tearDown(self):
         self.curl.close()
-    
+
     # CURLOPT_CERTINFO was introduced in libcurl-7.19.1
     @util.min_libcurl(7, 19, 1)
     def test_certinfo_option(self):
         assert hasattr(pycurl, 'OPT_CERTINFO')
-    
+
     # CURLOPT_CERTINFO was introduced in libcurl-7.19.1
     @util.min_libcurl(7, 19, 1)
     @util.only_ssl
@@ -34,10 +34,10 @@ class CertinfoTest(unittest.TestCase):
         self.curl.setopt(pycurl.SSL_VERIFYPEER, 0)
         self.curl.perform()
         assert sio.getvalue().decode() == 'success'
-        
+
         certinfo = self.curl.getinfo(pycurl.INFO_CERTINFO)
         self.assertEqual([], certinfo)
-    
+
     # CURLOPT_CERTINFO was introduced in libcurl-7.19.1
     @util.min_libcurl(7, 19, 1)
     @util.only_ssl
@@ -45,7 +45,7 @@ class CertinfoTest(unittest.TestCase):
         # CURLOPT_CERTINFO only works with OpenSSL
         if 'openssl' not in pycurl.version.lower():
             raise nose.plugins.skip.SkipTest('libcurl does not use openssl')
-        
+
         self.curl.setopt(pycurl.URL, 'https://localhost:8383/success')
         sio = util.BytesIO()
         self.curl.setopt(pycurl.WRITEFUNCTION, sio.write)
@@ -54,7 +54,7 @@ class CertinfoTest(unittest.TestCase):
         self.curl.setopt(pycurl.SSL_VERIFYPEER, 0)
         self.curl.perform()
         assert sio.getvalue().decode() == 'success'
-        
+
         certinfo = self.curl.getinfo(pycurl.INFO_CERTINFO)
         # self signed certificate, one certificate in chain
         assert len(certinfo) == 1
