@@ -5,6 +5,7 @@
 import pycurl
 import unittest
 import select
+import flaky
 
 from . import appmanager
 from . import util
@@ -23,6 +24,7 @@ def teardown_module(mod):
     teardown_module_2(mod)
     teardown_module_1(mod)
 
+@flaky.flaky(max_runs=3)
 class MultiSocketSelectTest(unittest.TestCase):
     def test_multi_socket_select(self):
         sockets = set()
@@ -38,7 +40,7 @@ class MultiSocketSelectTest(unittest.TestCase):
         ]
 
         socket_events = []
-        
+
         # socket callback
         def socket(event, socket, multi, data):
             if event == pycurl.POLL_REMOVE:
@@ -55,7 +57,7 @@ class MultiSocketSelectTest(unittest.TestCase):
         m.setopt(pycurl.M_SOCKETFUNCTION, socket)
         m.handles = []
         for url in urls:
-            c = pycurl.Curl()
+            c = util.DefaultCurl()
             # save info in standard Python attributes
             c.url = url
             c.body = util.BytesIO()
@@ -67,11 +69,11 @@ class MultiSocketSelectTest(unittest.TestCase):
             m.add_handle(c)
 
         # get data
-        num_handles = len(m.handles)
+        #num_handles = len(m.handles)
 
         while (pycurl.E_CALL_MULTI_PERFORM==m.socket_all()[0]):
             pass
-            
+
         timeout = m.timeout()
 
         # timeout might be -1, indicating that all work is done
@@ -101,18 +103,18 @@ class MultiSocketSelectTest(unittest.TestCase):
         for c in m.handles:
             self.assertEqual('success', c.body.getvalue().decode())
             self.assertEqual(200, c.http_code)
-            
+
             # multi, not curl handle
             self.check(pycurl.POLL_IN, m, socket_events)
             self.check(pycurl.POLL_REMOVE, m, socket_events)
-        
+
         # close handles
         for c in m.handles:
             # pycurl API calls
             m.remove_handle(c)
             c.close()
         m.close()
-    
+
     def check(self, event, multi, socket_events):
         for event_, multi_ in socket_events:
             if event == event_ and multi == multi_:
