@@ -12,7 +12,7 @@ setup_module, teardown_module = appmanager.setup(('app', 8380))
 
 class ResetTest(unittest.TestCase):
     def test_reset(self):
-        c = pycurl.Curl()
+        c = util.DefaultCurl()
         c.setopt(pycurl.USERAGENT, 'Phony/42')
         c.setopt(pycurl.URL, 'http://localhost:8380/header?h=user-agent')
         sio = util.BytesIO()
@@ -20,7 +20,7 @@ class ResetTest(unittest.TestCase):
         c.perform()
         user_agent = sio.getvalue().decode()
         assert user_agent == 'Phony/42'
-        
+
         c.reset()
         c.setopt(pycurl.URL, 'http://localhost:8380/header?h=user-agent')
         sio = util.BytesIO()
@@ -30,13 +30,13 @@ class ResetTest(unittest.TestCase):
         # we also check that the request succeeded after curl
         # object has been reset
         assert user_agent.startswith('PycURL')
-    
+
     # XXX this test was broken when it was test_reset.py
     def skip_reset_with_multi(self):
         outf = util.BytesIO()
         cm = pycurl.CurlMulti()
 
-        eh = pycurl.Curl()
+        eh = util.DefaultCurl()
 
         for x in range(1, 20):
             eh.setopt(pycurl.WRITEFUNCTION, outf.write)
