@@ -6,10 +6,6 @@ import threading
 import pycurl
 import unittest
 import nose.tools
-try:
-    import urllib.parse as urllib_parse
-except ImportError:
-    import urllib as urllib_parse
 
 from . import appmanager
 from . import util
@@ -20,7 +16,7 @@ class WorkerThread(threading.Thread):
 
     def __init__(self, share):
         threading.Thread.__init__(self)
-        self.curl = pycurl.Curl()
+        self.curl = util.DefaultCurl()
         self.curl.setopt(pycurl.URL, 'http://localhost:8380/success')
         self.curl.setopt(pycurl.SHARE, share)
         self.sio = util.BytesIO()
@@ -42,30 +38,30 @@ class ShareTest(unittest.TestCase):
 
         t1.start()
         t2.start()
-        
+
         t1.join()
         t2.join()
-        
+
         del s
-        
+
         self.assertEqual('success', t1.sio.getvalue().decode())
         self.assertEqual('success', t2.sio.getvalue().decode())
-    
+
     def test_share_close(self):
         s = pycurl.CurlShare()
         s.close()
-    
+
     def test_share_close_twice(self):
         s = pycurl.CurlShare()
         s.close()
         s.close()
-    
+
     # positional arguments are rejected
     @nose.tools.raises(TypeError)
     def test_positional_arguments(self):
-        c = pycurl.CurlShare(1)
-    
+        pycurl.CurlShare(1)
+
     # keyword arguments are rejected
     @nose.tools.raises(TypeError)
     def test_keyword_arguments(self):
-        c = pycurl.CurlShare(a=1)
+        pycurl.CurlShare(a=1)
