@@ -13,18 +13,18 @@ setup_module, teardown_module = appmanager.setup(('app', 8380))
 
 class WriteToStringioTest(unittest.TestCase):
     def setUp(self):
-        self.curl = pycurl.Curl()
-    
+        self.curl = util.DefaultCurl()
+
     def tearDown(self):
         self.curl.close()
-    
+
     def test_write_to_bytesio(self):
         self.curl.setopt(pycurl.URL, 'http://localhost:8380/success')
         sio = util.BytesIO()
         self.curl.setopt(pycurl.WRITEFUNCTION, sio.write)
         self.curl.perform()
         self.assertEqual('success', sio.getvalue().decode())
-    
+
     @util.only_python3
     def test_write_to_stringio(self):
         self.curl.setopt(pycurl.URL, 'http://localhost:8380/success')
@@ -33,7 +33,7 @@ class WriteToStringioTest(unittest.TestCase):
         self.curl.setopt(pycurl.WRITEFUNCTION, sio.write)
         try:
             self.curl.perform()
-            
+
             self.fail('Should have received a write error')
         except pycurl.error:
             err, msg = sys.exc_info()[1].args
