@@ -1,6 +1,7 @@
 # Bootstrap python binary:
-# http://www.python.org/ftp/python/3.3.4/python-3.3.4.msi
-# http://www.python.org/ftp/python/3.3.4/python-3.3.4.amd64.msi
+# http://www.python.org/ftp/python/3.3.5/python-3.3.5.msi
+# Then execute:
+# msiexec /i c:\dev\build-pycurl\archives\python-3.3.5.msi /norestart /passive InstallAllUsers=1 Include_test=0 Include_doc=0 Include_launcher=0 Include_ckltk=0 TargetDir=c:\dev\32\python33
 # msvc9/vs2008 express:
 # http://go.microsoft.com/?linkid=7729279
 # msvc10/vs2010 express:
@@ -31,59 +32,85 @@ root = 'c:/dev/build-pycurl'
 # where msysgit is installed
 git_root = 'c:/program files/git'
 # where NASM is installed, for building OpenSSL
-nasm_path = 'c:/program files (x86)/nasm'
+nasm_path = ('c:/dev/nasm', 'c:/program files/nasm', 'c:/program files (x86)/nasm')
 # where ActiveState Perl is installed, for building 64-bit OpenSSL
-activestate_perl_path = r'c:\dev\perl64'
+activestate_perl_path = ('c:/perl64', r'c:\dev\perl64')
 # which versions of python to build against
-python_versions = ['2.6.6', '2.7.10', '3.2.5', '3.3.5', '3.4.3', '3.5.0']
+python_versions = ['2.6.6', '2.7.10', '3.2.5', '3.3.5', '3.4.3', '3.5.4', '3.6.2']
 # where pythons are installed
-python_path_template = 'c:/dev/%(bitness)s/python%(python_version)s/python'
+python_path_template = 'c:/dev/%(bitness)s/python%(python_release)s/python'
 vc_paths = {
-    # where msvc 9 is installed, for python 2.6 through 3.2
+    # where msvc 9/vs 2008 is installed, for python 2.6 through 3.2
     'vc9': None,
-    # where msvc 10 is installed, for python 3.3 through 3.4
+    # where msvc 10/vs 2010 is installed, for python 3.3 through 3.4
     'vc10': None,
-    # where msvc 14 is installed, for python 3.5
+    # where msvc 14/vs 2015 is installed, for python 3.5
     'vc14': None,
 }
 # whether to link libcurl against zlib
 use_zlib = True
 # which version of zlib to use, will be downloaded from internet
-zlib_version = '1.2.8'
+zlib_version = '1.2.11'
 # whether to use openssl instead of winssl
 use_openssl = True
 # which version of openssl to use, will be downloaded from internet
-openssl_version = '1.0.2e'
+openssl_version = '1.1.0g'
 # whether to use c-ares
 use_cares = True
-cares_version = '1.10.0'
+cares_version = '1.13.0'
 # whether to use libssh2
 use_libssh2 = True
-libssh2_version = '1.6.0'
+libssh2_version = '1.8.0'
 # which version of libcurl to use, will be downloaded from internet
-libcurl_version = '7.46.0'
+libcurl_version = '7.57.0'
+# virtualenv version
+virtualenv_version = '15.1.0'
+# whether to build binary wheels
+build_wheels = True
 # pycurl version to build, we should know this ourselves
-pycurl_version = '7.21.5'
+pycurl_version = '7.43.0.1'
 
 default_vc_paths = {
-    # where msvc 9 is installed, for python 2.6 through 3.2
+    # where msvc 9 is installed, for python 2.6-3.2
     'vc9': [
         'c:/program files (x86)/microsoft visual studio 9.0',
         'c:/program files/microsoft visual studio 9.0',
     ],
-    # where msvc 10 is installed, for python 3.3 through 3.4
+    # where msvc 10 is installed, for python 3.3-3.4
     'vc10': [
         'c:/program files (x86)/microsoft visual studio 10.0',
         'c:/program files/microsoft visual studio 10.0',
     ],
-    # where msvc 14 is installed, for python 3.5
+    # where msvc 14 is installed, for python 3.5-3.6
     'vc14': [
         'c:/program files (x86)/microsoft visual studio 14.0',
         'c:/program files/microsoft visual studio 14.0',
     ],
 }
 
-import os, os.path, sys, subprocess, shutil, contextlib, zipfile
+def short_python_versions(python_versions):
+    return ['.'.join(python_version.split('.')[:2])
+        for python_version in python_versions]
+
+def needed_vc_versions(python_versions):
+    return [vc_version for vc_version in vc_paths.keys()
+        if vc_version in [
+            python_vc_versions[short_python_version]
+            for short_python_version in short_python_versions(python_versions)]]
+
+import os, os.path, sys, subprocess, shutil, contextlib, zipfile, re
+
+def select_existing_path(paths):
+    if isinstance(paths, list) or isinstance(paths, tuple):
+        for path in paths:
+            if os.path.exists(path):
+                return path
+        return paths[0]
+    else:
+        return paths
+
+nasm_path = select_existing_path(nasm_path)
+activestate_perl_path = select_existing_path(activestate_perl_path)
 
 archives_path = os.path.join(root, 'archives')
 state_path = os.path.join(root, 'state')
@@ -100,20 +127,31 @@ python_vc_versions = {
     '3.3': 'vc10',
     '3.4': 'vc10',
     '3.5': 'vc14',
+    '3.6': 'vc14',
 }
 vc_versions = vc_paths.keys()
 dir_here = os.path.abspath(os.path.dirname(__file__))
 
+openssl_version_tuple = tuple(
+    int(part) if part < 'a' else part
+    for part in re.sub(r'([a-z])', r'.\1', openssl_version).split('.')
+)
+
 try:
     from urllib.request import urlopen
 except ImportError:
     from urllib import urlopen
 
+def mkdir_p(path):
+    if not os.path.exists(path):
+        os.makedirs(path)
+
 def fetch(url, archive=None):
     if archive is None:
         archive = os.path.basename(url)
     if not os.path.exists(archive):
         sys.stdout.write("Fetching %s\n" % url)
+        sys.stdout.flush()
         io = urlopen(url)
         tmp_path = os.path.join(os.path.dirname(archive),
             '.%s.part' % os.path.basename(archive))
@@ -125,6 +163,11 @@ def fetch(url, archive=None):
                 f.write(chunk)
         os.rename(tmp_path, archive)
 
+def fetch_to_archives(url):
+    mkdir_p(archives_path)
+    path = os.path.join(archives_path, os.path.basename(url))
+    fetch(url, path)
+
 @contextlib.contextmanager
 def in_dir(dir):
     old_cwd = os.getcwd()
@@ -138,8 +181,7 @@ def in_dir(dir):
 def step(step_fn, args, target_dir):
     #step = step_fn.__name__
     state_tag = target_dir
-    if not os.path.exists(state_path):
-        os.makedirs(state_path)
+    mkdir_p(state_path)
     state_file_path = os.path.join(state_path, state_tag)
     if not os.path.exists(state_file_path) or not os.path.exists(target_dir):
         step_fn(*args)
@@ -158,6 +200,31 @@ def rename_for_vc(basename, suffix):
     os.rename(basename, suffixed_dir)
     return suffixed_dir
 
+class PythonRelease(str):
+    @property
+    def dotless(self):
+        return self.replace('.', '')
+
+class PythonVersion(str):
+    @property
+    def release(self):
+        return PythonRelease('.'.join(self.split('.')[:2]))
+
+def python_releases():
+    return [PythonRelease('.'.join(version.split('.')[:2]))
+        for version in python_versions]
+
+class PythonBinary(object):
+    def __init__(self, python_release, bitness):
+        self.python_release = python_release
+        self.bitness = bitness
+
+    @property
+    def executable_path(self):
+        return python_path_template % dict(
+            python_release=self.python_release.dotless,
+            bitness=self.bitness)
+
 class Builder(object):
     def __init__(self, **kwargs):
         bitness = kwargs.pop('bitness')
@@ -211,14 +278,30 @@ class Builder(object):
     def execute_batch(self):
         with open('doit.bat', 'w') as f:
             f.write(self.vcvars_cmd)
+            if self.vc_version == 'vc14':
+                # I don't know why vcvars doesn't configure this under vc14
+                windows_sdk_path = 'c:\\program files (x86)\\microsoft sdks\\windows\\v7.1a'
+                f.write('set include=%s\\include;%%include%%\n' % windows_sdk_path)
+                if self.bitness == 32:
+                    f.write('set lib=%s\\lib;%%lib%%\n' % windows_sdk_path)
+                    f.write('set path=%s\\bin;%%path%%\n' % windows_sdk_path)
+                else:
+                    f.write('set lib=%s\\lib\\x64;%%lib%%\n' % windows_sdk_path)
+                    f.write('set path=%s\\bin\\x64;%%path%%\n' % windows_sdk_path)
             f.write(self.nasm_cmd)
             yield f
-        if True:
+        if False:
             print("Executing:")
             with open('doit.bat', 'r') as f:
                 print(f.read())
             sys.stdout.flush()
-        subprocess.check_call(['doit.bat'])
+        rv = subprocess.call(['doit.bat'])
+        if rv != 0:
+            print("\nFailed to execute the following commands:\n")
+            with open('doit.bat', 'r') as f:
+                print(f.read())
+            sys.stdout.flush()
+            exit(3)
 
     @property
     def vc_tag(self):
@@ -287,25 +370,56 @@ class OpensslBuilder(Builder):
         openssl_dir = rename_for_vc('openssl-%s' % self.openssl_version, self.vc_tag)
         with in_dir(openssl_dir):
             with self.execute_batch() as f:
-                f.write("patch -p0 < %s\n" % os.path.join(dir_here, 'winbuild', 'openssl-fix-crt.patch'))
+                if openssl_version_tuple < (1, 1):
+                    # openssl 1.0.2
+                    f.write("patch -p0 < %s\n" % os.path.join(dir_here, 'winbuild', 'openssl-fix-crt-1.0.2.patch'))
+                else:
+                    # openssl 1.1.0
+                    f.write("patch -p0 < %s\n" % os.path.join(dir_here, 'winbuild', 'openssl-fix-crt-1.1.0.patch'))
                 if self.bitness == 64:
                     target = 'VC-WIN64A'
                     batch_file = 'do_win64a'
-
-                    # msysgit perl has trouble with backslashes used in
-                    # win64 assembly things; use ActiveState Perl
-                    f.write("set path=%s;%%path%%\n" % activestate_perl_bin_path)
-                    f.write("perl -v\n")
                 else:
                     target = 'VC-WIN32'
                     batch_file = 'do_nasm'
+
+                # msysgit perl has trouble with backslashes used in
+                # win64 assembly things in openssl 1.0.2
+                # and in x86 assembly as well in openssl 1.1.0;
+                # use ActiveState Perl
+                if not os.path.exists(activestate_perl_bin_path):
+                    raise ValueError('activestate_perl_bin_path refers to a nonexisting path')
+                if not os.path.exists(os.path.join(activestate_perl_bin_path, 'perl.exe')):
+                    raise ValueError('No perl binary in activestate_perl_bin_path')
+                f.write("set path=%s;%%path%%\n" % activestate_perl_bin_path)
+                f.write("perl -v\n")
+
                 openssl_prefix = os.path.join(os.path.realpath('.'), 'build')
                 # Do not want compression:
                 # https://en.wikipedia.org/wiki/CRIME
-                f.write("perl Configure %s no-comp --prefix=%s\n" % (target, openssl_prefix))
-                f.write("call ms\\%s\n" % batch_file)
-                f.write("nmake -f ms\\nt.mak\n")
-                f.write("nmake -f ms\\nt.mak install\n")
+                extras = ['no-comp']
+                if openssl_version_tuple >= (1, 1):
+                    # openssl 1.1.0
+                    # in 1.1.0 the static/shared selection is handled by
+                    # invoking the right makefile
+                    extras += ['no-shared']
+                    
+                    # looks like openssl 1.1.0c does not derive
+                    # --openssldir from --prefix, like its Configure claims,
+                    # and like 1.0.2 does; provide a relative openssl dir
+                    # manually
+                    extras += ['--openssldir=ssl']
+                f.write("perl Configure %s %s --prefix=%s\n" % (target, ' '.join(extras), openssl_prefix))
+                
+                if openssl_version_tuple < (1, 1):
+                    # openssl 1.0.2
+                    f.write("call ms\\%s\n" % batch_file)
+                    f.write("nmake -f ms\\nt.mak\n")
+                    f.write("nmake -f ms\\nt.mak install\n")
+                else:
+                    # openssl 1.1.0
+                    f.write("nmake\n")
+                    f.write("nmake install\n")
 
     @property
     def output_dir_path(self):
@@ -335,10 +449,16 @@ class CaresBuilder(Builder):
     def build(self):
         fetch('http://c-ares.haxx.se/download/c-ares-%s.tar.gz' % (self.cares_version))
         untar('c-ares-%s' % self.cares_version)
+        if self.cares_version == '1.12.0':
+            # msvc_ver.inc is missing in c-ares-1.12.0.tar.gz
+            # https://github.com/c-ares/c-ares/issues/69
+            fetch('https://raw.githubusercontent.com/c-ares/c-ares/cares-1_12_0/msvc_ver.inc',
+                  archive='c-ares-1.12.0/msvc_ver.inc')
         cares_dir = rename_for_vc('c-ares-%s' % self.cares_version, self.vc_tag)
         with in_dir(cares_dir):
             with self.execute_batch() as f:
-                f.write("patch -p1 < %s\n" % os.path.join(dir_here, 'winbuild', 'c-ares-vs2015.patch'))
+                if self.cares_version == '1.10.0':
+                    f.write("patch -p1 < %s\n" % os.path.join(dir_here, 'winbuild', 'c-ares-vs2015.patch'))
                 f.write("nmake -f Makefile.msvc\n")
 
     @property
@@ -440,7 +560,7 @@ class LibcurlBuilder(Builder):
         return 'curl-%s-%s' % (self.libcurl_version, self.vc_tag)
 
     def build(self):
-        fetch('http://curl.haxx.se/download/curl-%s.tar.gz' % self.libcurl_version)
+        fetch('https://curl.haxx.se/download/curl-%s.tar.gz' % self.libcurl_version)
         untar('curl-%s' % self.libcurl_version)
         curl_dir = rename_for_vc('curl-%s' % self.libcurl_version, self.vc_tag)
         with in_dir(os.path.join(curl_dir, 'winbuild')):
@@ -471,6 +591,12 @@ class LibcurlBuilder(Builder):
                     f.write("set include=%%include%%;%s\n" % libssh2_builder.include_path)
                     f.write("set lib=%%lib%%;%s\n" % libssh2_builder.lib_path)
                     extra_options += ' WITH_SSH2=%s' % dll_or_static
+                if openssl_version_tuple >= (1, 1):
+                    # openssl 1.1.0
+                    # https://curl.haxx.se/mail/lib-2016-08/0104.html
+                    # https://github.com/curl/curl/issues/984
+                    # crypt32.lib: http://stackoverflow.com/questions/37522654/linking-with-openssl-lib-statically
+                    extra_options += ' MAKE="NMAKE /e" SSL_LIBS="libssl.lib libcrypto.lib crypt32.lib"'
                 f.write("nmake /f Makefile.vc ENABLE_IDN=no%s\n" % extra_options)
 
     @property
@@ -522,8 +648,8 @@ class LibcurlBuilder(Builder):
 
 class PycurlBuilder(Builder):
     def __init__(self, **kwargs):
-        self.python_version = kwargs.pop('python_version')
-        kwargs['vc_version'] = python_vc_versions[self.python_version]
+        self.python_release = kwargs.pop('python_release')
+        kwargs['vc_version'] = python_vc_versions[self.python_release]
         super(PycurlBuilder, self).__init__(**kwargs)
         self.pycurl_version = kwargs.pop('pycurl_version')
         self.libcurl_version = kwargs.pop('libcurl_version')
@@ -538,9 +664,11 @@ class PycurlBuilder(Builder):
 
     @property
     def python_path(self):
-        return python_path_template % dict(
-            python_version=self.python_version.replace('.', ''),
-            bitness=self.bitness)
+        if build_wheels:
+            python_path = os.path.join(archives_path, 'venv-%s-%s' % (self.python_release, self.bitness), 'scripts', 'python')
+        else:
+            python_path = PythonBinary(self.python_release, self.bitness).executable_path
+        return python_path
 
     @property
     def platform_indicator(self):
@@ -570,10 +698,9 @@ class PycurlBuilder(Builder):
         dll_paths = [os.path.abspath(dll_path) for dll_path in dll_paths]
         with in_dir(os.path.join('pycurl-%s' % self.pycurl_version)):
             dest_lib_path = 'build/lib.%s-%s' % (self.platform_indicator,
-                self.python_version)
-            if not os.path.exists(dest_lib_path):
-                # exists for building additional targets for the same python version
-                os.makedirs(dest_lib_path)
+                self.python_release)
+            # exists for building additional targets for the same python version
+            mkdir_p(dest_lib_path)
             if self.use_dlls:
                 for dll_path in dll_paths:
                     shutil.copy(dll_path, dest_lib_path)
@@ -585,22 +712,30 @@ class PycurlBuilder(Builder):
                     libcurl_arg = '--libcurl-lib-name=libcurl_a.lib'
                 if self.use_openssl:
                     libcurl_arg += ' --with-openssl'
+                    if openssl_version_tuple >= (1, 1):
+                        libcurl_arg += ' --openssl-lib-name=""'
                     openssl_builder = OpensslBuilder(bitness=self.bitness, vc_version=self.vc_version, openssl_version=self.openssl_version)
                     f.write("set include=%%include%%;%s\n" % openssl_builder.include_path)
                     f.write("set lib=%%lib%%;%s\n" % openssl_builder.lib_path)
-                    f.write("%s setup.py %s --curl-dir=%s %s\n" % (
+                #if build_wheels:
+                    #f.write("call %s\n" % os.path.join('..', 'venv-%s-%s' % (self.python_release, self.bitness), 'Scripts', 'activate'))
+                if build_wheels:
+                    targets = targets + ['bdist_wheel']
+                f.write("%s setup.py %s --curl-dir=%s %s\n" % (
                     self.python_path, ' '.join(targets), libcurl_dir, libcurl_arg))
             if 'bdist' in targets:
                 zip_basename_orig = 'pycurl-%s.%s.zip' % (
                     self.pycurl_version, self.platform_indicator)
                 zip_basename_new = 'pycurl-%s.%s-py%s.zip' % (
-                    self.pycurl_version, self.platform_indicator, self.python_version)
+                    self.pycurl_version, self.platform_indicator, self.python_release)
                 with zipfile.ZipFile('dist/%s' % zip_basename_orig, 'r') as src_zip:
                     with zipfile.ZipFile('dist/%s' % zip_basename_new, 'w') as dest_zip:
                         for name in src_zip.namelist():
                             parts = name.split('/')
-                            while parts[0] != 'python%s' % self.python_version.replace('.', ''):
-                                parts.pop(0)
+                            while True:
+                                popped = parts.pop(0)
+                                if popped == 'python%s' % self.python_release.dotless or popped.startswith('venv-'):
+                                    break
                             assert len(parts) > 0
                             new_name = '/'.join(parts)
                             print('Recompressing %s -> %s' % (name, new_name))
@@ -608,7 +743,7 @@ class PycurlBuilder(Builder):
                             member = src_zip.open(name)
                             dest_zip.writestr(new_name, member.read(), zipfile.ZIP_DEFLATED)
 
-def build_dependencies():
+def build_dependencies(bitnesses=(32, 64)):
     if use_libssh2:
         if not use_zlib:
             # technically we can build libssh2 without zlib but I don't want to bother
@@ -618,11 +753,12 @@ def build_dependencies():
 
     if git_bin_path:
         os.environ['PATH'] += ";%s" % git_bin_path
-    if not os.path.exists(archives_path):
-        os.makedirs(archives_path)
+    mkdir_p(archives_path)
     with in_dir(archives_path):
-        for bitness in (32, 64):
-            for vc_version in vc_versions:
+        for bitness in bitnesses:
+            for vc_version in needed_vc_versions(python_versions):
+                if opts.verbose:
+                    print('Builddep for %s, %s-bit' % (vc_version, bitness))
                 if use_zlib:
                     zlib_builder = ZlibBuilder(bitness=bitness, vc_version=vc_version, zlib_version=zlib_version)
                     step(zlib_builder.build, (), zlib_builder.state_tag)
@@ -643,27 +779,33 @@ def build_dependencies():
                     libcurl_version=libcurl_version)
                 step(libcurl_builder.build, (), libcurl_builder.state_tag)
 
+bitnesses = (32, 64)
+
 def build():
     # note: adds git_bin_path to PATH if necessary, and creates archives_path
-    build_dependencies()
+    build_dependencies(bitnesses)
     with in_dir(archives_path):
         def prepare_pycurl():
-            #fetch('http://pycurl.sourceforge.net/download/pycurl-%s.tar.gz' % pycurl_version)
+            #fetch('https://dl.bintray.com/pycurl/pycurl/pycurl-%s.tar.gz' % pycurl_version)
             if os.path.exists('pycurl-%s' % pycurl_version):
                 #shutil.rmtree('pycurl-%s' % pycurl_version)
                 subprocess.check_call([rm_path, '-rf', 'pycurl-%s' % pycurl_version])
             #subprocess.check_call([tar_path, 'xf', 'pycurl-%s.tar.gz' % pycurl_version])
             shutil.copytree('c:/dev/pycurl', 'pycurl-%s' % pycurl_version)
+            if build_wheels:
+                with in_dir('pycurl-%s' % pycurl_version):
+                    subprocess.check_call(['sed', '-i',
+                        's/from distutils.core import setup/from setuptools import setup/',
+                        'setup.py'])
 
         prepare_pycurl()
 
-        for bitness in (32, 64):
-            python_releases = ['.'.join(version.split('.')[:2]) for version in python_versions]
-            for python_version in python_releases:
+        for bitness in bitnesses:
+            for python_release in python_releases():
                 targets = ['bdist', 'bdist_wininst', 'bdist_msi']
-                vc_version = python_vc_versions[python_version]
+                vc_version = python_vc_versions[python_release]
                 builder = PycurlBuilder(bitness=bitness, vc_version=vc_version,
-                    python_version=python_version, pycurl_version=pycurl_version,
+                    python_release=python_release, pycurl_version=pycurl_version,
                     use_zlib=use_zlib, zlib_version=zlib_version,
                     use_openssl=use_openssl, openssl_version=openssl_version,
                     use_cares=use_cares, cares_version=cares_version,
@@ -671,7 +813,8 @@ def build():
                     libcurl_version=libcurl_version)
                 builder.build(targets)
 
-def download_pythons():
+def python_metas():
+    metas = []
     for version in python_versions:
         parts = [int(part) for part in version.split('.')]
         if parts[0] >= 3 and parts[1] >= 5:
@@ -680,25 +823,132 @@ def download_pythons():
         else:
             ext = 'msi'
             amd64_suffix = '.amd64'
-        url = 'https://www.python.org/ftp/python/%s/python-%s.%s' % (version, version, ext)
-        fetch(url, os.path.join(archives_path, 'python-%s.%s') % (version, ext))
-        url = 'https://www.python.org/ftp/python/%s/python-%s%s.%s' % (version, version, amd64_suffix, ext)
-        fetch(url, os.path.join(archives_path, 'python-%s%s.%s') % (version, amd64_suffix, ext))
+        url_32 = 'https://www.python.org/ftp/python/%s/python-%s.%s' % (version, version, ext)
+        url_64 = 'https://www.python.org/ftp/python/%s/python-%s%s.%s' % (version, version, amd64_suffix, ext)
+        meta = dict(
+            version=version, ext=ext, amd64_suffix=amd64_suffix,
+            url_32=url_32, url_64=url_64,
+            installed_path_32 = 'c:\\dev\\32\\python%d%d' % (parts[0], parts[1]),
+            installed_path_64 = 'c:\\dev\\64\\python%d%d' % (parts[0], parts[1]),
+        )
+        metas.append(meta)
+    return metas
+
+def download_pythons():
+    for meta in python_metas():
+        for bitness in bitnesses:
+            fetch_to_archives(meta['url_%d' % bitness])
+
+def install_pythons():
+    for meta in python_metas():
+        for bitness in bitnesses:
+            if not os.path.exists(meta['installed_path_%d' % bitness]):
+                install_python(meta, bitness)
+
+def fix_slashes(path):
+    return path.replace('/', '\\')
+
+# http://eddiejackson.net/wp/?p=10276
+def install_python(meta, bitness):
+    archive_path = fix_slashes(os.path.join(archives_path, os.path.basename(meta['url_%d' % bitness])))
+    if meta['ext'] == 'exe':
+        cmd = [archive_path]
+    else:
+        cmd = ['msiexec', '/i', archive_path, '/norestart']
+    cmd += ['/passive', 'InstallAllUsers=1',
+            'Include_test=0', 'Include_doc=0', 'Include_launcher=0',
+            'Include_ckltk=0',
+            'TargetDir=%s' % meta['installed_path_%d' % bitness],
+        ]
+    sys.stdout.write('Installing python %s (%d bit)\n' % (meta['version'], bitness))
+    print(' '.join(cmd))
+    sys.stdout.flush()
+    subprocess.check_call(cmd)
 
 def download_bootstrap_python():
     version = python_versions[-2]
     url = 'https://www.python.org/ftp/python/%s/python-%s.msi' % (version, version)
     fetch(url)
 
-if len(sys.argv) > 1:
-    if sys.argv[1] == 'download':
+def install_virtualenv():
+    with in_dir(archives_path):
+        #fetch('https://pypi.python.org/packages/source/v/virtualenv/virtualenv-%s.tar.gz' % virtualenv_version)
+        fetch('https://pypi.python.org/packages/d4/0c/9840c08189e030873387a73b90ada981885010dd9aea134d6de30cd24cb8/virtualenv-15.1.0.tar.gz')
+        for bitness in bitnesses:
+            for python_release in python_releases():
+                print('Installing virtualenv %s for Python %s (%s bit)' % (virtualenv_version, python_release, bitness))
+                sys.stdout.flush()
+                untar('virtualenv-%s' % virtualenv_version)
+                with in_dir('virtualenv-%s' % virtualenv_version):
+                    python_binary = PythonBinary(python_release, bitness)
+                    cmd = [python_binary.executable_path, 'setup.py', 'install']
+                    subprocess.check_call(cmd)
+
+def create_virtualenvs():
+    for bitness in bitnesses:
+        for python_release in python_releases():
+            print('Creating a virtualenv for Python %s (%s bit)' % (python_release, bitness))
+            sys.stdout.flush()
+            with in_dir(archives_path):
+                python_binary = PythonBinary(python_release, bitness)
+                venv_basename = 'venv-%s-%s' % (python_release, bitness)
+                cmd = [python_binary.executable_path, '-m', 'virtualenv', venv_basename]
+                subprocess.check_call(cmd)
+
+import optparse
+
+parser = optparse.OptionParser()
+parser.add_option('-b', '--bitness', help='Bitnesses build for, comma separated')
+parser.add_option('-p', '--python', help='Python versions to build for, comma separated')
+parser.add_option('-v', '--verbose', help='Print what is being done', action='store_true')
+opts, args = parser.parse_args()
+
+if opts.bitness:
+    chosen_bitnesses = [int(bitness) for bitness in opts.bitness.split(',')]
+    for bitness in chosen_bitnesses:
+        if bitness not in bitnesses:
+            print('Invalid bitness %d' % bitness)
+            exit(2)
+    bitnesses = chosen_bitnesses
+
+if opts.python:
+    chosen_pythons = opts.python.split(',')
+    chosen_python_versions = []
+    for python in chosen_pythons:
+        python = python.replace('.', '')
+        python = python[0] + '.' + python[1] + '.'
+        ok = False
+        for python_version in python_versions:
+            if python_version.startswith(python):
+                chosen_python_versions.append(python_version)
+                ok = True
+        if not ok:
+            print('Invalid python %s' % python)
+            exit(2)
+    python_versions = chosen_python_versions
+
+# https://stackoverflow.com/questions/35569042/python-3-ssl-certificate-verify-failed
+import ssl
+try:
+	ssl._create_default_https_context = ssl._create_unverified_context
+except AttributeError:
+	pass
+
+if len(args) > 0:
+    if args[0] == 'download':
         download_pythons()
-    elif sys.argv[1] == 'bootstrap':
+    elif args[0] == 'bootstrap':
         download_bootstrap_python()
-    elif sys.argv[1] == 'builddeps':
-        build_dependencies()
+    elif args[0] == 'installpy':
+        install_pythons()
+    elif args[0] == 'builddeps':
+        build_dependencies(bitnesses)
+    elif args[0] == 'installvirtualenv':
+        install_virtualenv()
+    elif args[0] == 'createvirtualenvs':
+        create_virtualenvs()
     else:
-        print('Unknown command: %s' % sys.argv[1])
+        print('Unknown command: %s' % args[0])
         exit(2)
 else:
     build()
